
' Demo code showing how to move through a database with and
' without an index. Without an index, movement through the
' database is based on the order the records are physically
' stored in the database. If an index is used then the
' database records are displayed based on the sort order
' of the keys in the index.

VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
	IMPORT "xCheetah"

DECLARE FUNCTION Entry ()

FUNCTION Entry ()

	DIM Fd$[2]

'define the names of the database & index
  DBFname$ = "Cheetah.dbf"
  IDXname$ = "Cheetah.idx"

'define the structure of the databases
  Fd$[0] = "CUSTID,N,5,0"     'Numeric field, length 5
  Fd$[1] = "CUSTNAME,C,35,0"  'Character field, length 35
  Fd$[2] = "SALARY,N,14,2"    'Numeric field, length 14, 2 decimal places (i.e "###########.##")

'create the database
  xdbCreate (DBFname$, @Fd$[])
  'to create a database using encyption and/or variable length memo fields then
  'use the xdbCreateEX function.
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating database.")
     xdbResetError ()
     RETURN
  END IF

'open the database (database must be open prior to creating index)
  dbHandle = xdbOpen (DBFname$)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " opening database.")
     xdbResetError ()
     RETURN
  END IF

'create the index (database must be open)
  IndexExpr$ = "UPPER(CUSTID)"   'index is not case sensitive
  Duplicates = $$XDBTRUE         'allow duplicate customer ID's

  xdbCreateIndex (@IDXname$, dbHandle, @IndexExpr$, Duplicates)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError()) + " creating index.")
     xdbResetError ()
     RETURN
  END IF

'open the index
  idxHandle = xdbOpenIndex (IDXname$, dbHandle)
  IF xdbError() THEN
     XstAlert ("Error: " + STR$(xdbError&) + " opening index.")
     xdbResetError ()
     RETURN
  END IF

'add records to the database and index
'NOTE: When adding a large number of records it is better to close
'the index, do the appends to the database and then open the
'index and reindex. For a few hundred records there should be no
'performance issues.

  NumRecs = 1000

	XstAlert ("Press 'Okay' to create " + STR$(NumRecs) + " database and index records.")

'predefine the field#'s for speed. If you already know the
'which field name corresponds to which field number then
'there is no need to do this. These functions provide
'really fast lookups.

  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")

  CustIDLength = xdbFieldLength(dbHandle, CustIDfield)

  FOR x = 1 TO NumRecs

     xdbClearBuffer(dbHandle)  'this will clear the record buffer
		 temp = XstRandomRange (10000, 99999)    'get a random 5 digit customer number

		 CustID$ = STRING$(temp)
     xdbAssignField(dbHandle, "", CustIDfield, @CustID$)
     xdbAssignField(dbHandle, "", CustNameField, @"The big Cat")
     xdbAssignField(dbHandle, "", SalaryField, "-1234.50")

     'add to the end of the database (Append) & add the key to the index.
     xdbAddRecord(dbHandle)

     IF xdbError() THEN
        XstAlert ( "Error:" + STR$(xdbError()) + " adding record.")
        EXIT FOR
     END IF

  NEXT x

	? "Database and Index records created."
  ? "Database Records : " + STR$(xdbRecordCount(dbHandle))

   '  Seek for a key value. IF the value is not found using an exact
   '  Seek then perform a partial seek afterwards.
   LookFor$ = "10000"

   '  Turn On exact Match Requirement
   ExactSearch = $$XDBTRUE

   '  Search the Index
   stat = xdbSeek(dbHandle, idxHandle, LookFor$)

   IF stat = 0 THEN
      '  No exact match found. Perform a Partial seek to get all
      '  customer ID's that start with "10".

      LookFor$ = "10"    'DO NOT use a search string like "10   "

      '  This Partial Seek will set 'stat = 1', allowing entry
      '  into the loop 'DO UNTIL stat = $$XDBFALSE' block.
      stat = xdbSeekPartial(dbHandle, idxHandle, LookFor$)
      ExactSearch = $$XDBFALSE
   ELSE
      '  Get the Record Number that Seek found an Exact Matching Key
      RecNum = xdbRecordNumber(dbHandle)

      '  Get the Key field Value of the Exact Matching Key
      KeyFound$ = xdbFieldValue$(dbHandle, "", 1)
   END IF

   NumFound = 0

   DO UNTIL stat = $$XDBFALSE
      NumFound = NumFound + 1

      IF ExactSearch = $$XDBTRUE THEN
         '   Locate any record with an exact key match
         stat = xdbSeekNext(dbHandle, idxHandle, LookFor$)
      ELSE

         DO WHILE stat <> 0  '  Loop when mutiple results are possible
            '   Locate any record with a partial key match
            stat = xdbSeekPartialNext(dbHandle, idxHandle, LookFor$)

            '  IF stat = 1, THEN Record Buffer now has a matching record
            IF stat > 0 THEN
               '  Increment Counter
               NumFound = NumFound + 1

               '  Get the Record Number that matched Partial Seek
               RecNum = xdbRecordNumber(dbHandle)

               '  Get the Key field Value of the partial match
               KeyFound$ = xdbFieldValue$(dbHandle, "", 1)

            END IF
         LOOP
      END IF
   LOOP


   IF ExactSearch = $$XDBTRUE THEN
      ? "Exact key search found" + STR$(NumFound) + " matches."
   ELSE
      ? "Partial key search found" + STR$(NumFound) + " matches."
   END IF

'close the database and related index
  xdbClose(dbHandle)
  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION
END PROGRAM